import argparse
import sys
import os

FILE_DIR = os.path.dirname(os.path.abspath(__file__))
BASE_DIR = os.path.join(FILE_DIR, "..")
sys.path.append(BASE_DIR)

from drippy.host_identity import HostIdentity
from drippy.drip_iroha_account import DripIrohaAccount

from json import dump

from dotenv.main import load_dotenv

def main(args):
  load_dotenv()
  parser = argparse.ArgumentParser(description="Generate a sorted list of geolocations from UA in the iroha blockchain.")
  parser.add_argument(
    "-f", "--file", 
    dest="host_identity_file", 
    metavar="host_identity_file.xml",
    type=str,
    help="Specify the host identity xml file."
  )
  parser.add_argument(
    "-i", "--iroha_key",
    dest="iroha_priv_key_file",
    metavar="iroha_private_key.priv",
    type=str,
    help="Specify the iroha private key file."
  )
  parser.add_argument(
    "-o", "--out",
    dest="outfile",
    metavar="output_file",
    type=str,
    help="Specify the output file for the path."
  )
  parser.add_argument(
    "--drone",
    type=str,
  )
  parser.add_argument(
    "--flight_id",
    type=str
  )

  args = parser.parse_args(args)
  hi_file = args.host_identity_file
  iroha_priv_key_file = args.iroha_priv_key_file
  outfile = os.path.join(FILE_DIR, args.outfile)

  if not os.path.isfile(hi_file) \
    or hi_file[-3:] != "xml":
    print("The specified file is not an xml file")
    sys.exit(-1)
  
  if not os.path.isfile(iroha_priv_key_file) \
    or iroha_priv_key_file[-4:] != "priv":
    print("The specified iroha private key file is not a .priv file")
    sys.exit(-1)
    
  hi = HostIdentity.from_file(hi_file)
  key = open(iroha_priv_key_file, 'r').read()
  iroha = DripIrohaAccount(key, hi.get_hhit(), hi.get_hid(), os.environ.get("IROHA_HOST"), os.environ.get("IROHA_PORT"))

  if not args.drone:
    account_id = input("Specify iroha account (hhit@domain): ")
  else:
    account_id = args.drone

  if not args.flight_id:
    flight_id = input("Specify the flight id: ")
  else:
    flight_id = args.flight_id

  geo_path = iroha.find_geo_path(account_id, flight_id)
  with open(outfile, 'w') as f:
    out_list = []
    for loc in geo_path:
      out_list.append(loc.get_geo_position())
    dump(out_list, f)

if __name__ == '__main__':
  main(sys.argv[1:])