import sys, getopt, code, os
from time import sleep
from binascii import hexlify

BASE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..")
sys.path.append(BASE_DIR)

from drippy.drip_iroha_account import DripIrohaAccount
from drippy.host_identity import HostIdentity
from drippy.util import get_timestamp

from datetime import datetime
import json

from dotenv.main import load_dotenv
load_dotenv()

HOST_IDENTITY_FILE = "/usr/local/etc/hip/my_host_identities.xml"
HOST = os.getenv("IROHA_HOST")
PORT = os.getenv("IROHA_PORT")

IDENTITIES_DIR = os.path.join(BASE_DIR, "identities")

ADMIN_HI_FILE = os.path.join(IDENTITIES_DIR, os.path.join("admin", "admin_identities.xml"))
ADMIN_IROHA_PRIV_KEY_FILE = os.path.join(IDENTITIES_DIR, os.path.join("admin", "200100250b3fa14c422d977815e6912e.priv"))

DRONE_HI_FILE = os.path.join(IDENTITIES_DIR, os.path.join("drone", "drone_identities.xml"))
DRONE_IROHA_PRIV_KEY_FILE = os.path.join(IDENTITIES_DIR, os.path.join("drone", "200100250b3fa14c1cee522e25add5b0.priv"))


def load_hi(hi_file):
  if hi_file == "":
    hi_file = HOST_IDENTITY_FILE
    
  hi = HostIdentity.from_file(hi_file)

  return hi

def load_iroha(iroha_priv_key_file, hit, hid):
  iroha_priv_key = ""
  with open(iroha_priv_key_file, 'r') as f:
    iroha_priv_key = f.read()
  iroha = DripIrohaAccount(iroha_priv_key, hit, hid, HOST, PORT)

  return iroha


if __name__ == '__main__':
  hi_admin = load_hi(ADMIN_HI_FILE)
  iroha_admin = load_iroha(ADMIN_IROHA_PRIV_KEY_FILE, hi_admin.get_hhit(), hi_admin.get_hid())

  hi_drone = load_hi(DRONE_HI_FILE)
  iroha_drone = load_iroha(DRONE_IROHA_PRIV_KEY_FILE, hi_drone.get_hhit(), hi_drone.get_hid())

  iroha_admin.set_account_details(
    "public_key",
    hi_admin.get_pub_key()
  )

  iroha_drone.set_account_details(
    "public_key",
    hi_drone.get_pub_key()
  )

  iroha_admin.set_account_details(
    "self_concise_attestation", 
    hexlify(hi_admin.generate_self_concise_attestation(get_timestamp(datetime(2022,1,1,0,0,0,0)))).decode("utf-8")
  )

  iroha_drone.set_account_details(
    "self_concise_attestation", 
    hexlify(hi_drone.generate_self_concise_attestation(get_timestamp(datetime(2022,1,1,0,0,0,0)))).decode("utf-8")
  )

  iroha_admin.set_account_details(
    "certificate", 
    hi_admin.generate_certificate(hi_drone, get_timestamp(datetime(2022,1,1,0,0,0,0))),
    iroha_drone.get_account_id()
  )
  
  iroha_admin.set_account_details(
    "public_key", 
    hi_admin.get_pub_key(),
    iroha_drone.get_account_id()
  )

  sleep(1) # wait until blocks have been created.
  print(json.dumps(iroha_admin.get_account(iroha_admin.get_account_id()), indent=2))
  print(json.dumps(iroha_admin.get_account(iroha_drone.get_account_id()), indent=2))
  